
import qgl.render

cdef extern from "GL/gl.h":
    ctypedef float GLfloat
    ctypedef float GLclampf
    ctypedef unsigned int GLenum
    ctypedef unsigned int GLbitfield
    ctypedef int GLint
    ctypedef unsigned int GLuint
    ctypedef int GLsizei
    ctypedef double GLdouble
    ctypedef double GLclampd

    int GL_CURRENT_BIT
    int GL_TEXTURE_BIT
    int GL_ENABLE_BIT
    int GL_LIGHTING_BIT
    int GL_POLYGON_BIT
    int GL_DEPTH_TEST
    int GL_LEQUAL
    int GL_CW
    int GL_SMOOTH
    int GL_BACK
    int GL_COLOR_BUFFER_BIT
    int GL_DEPTH_BUFFER_BIT
    int GL_BLEND
    int GL_SRC_ALPHA
    int GL_ONE_MINUS_SRC_ALPHA
    int GL_TEXTURE_2D
    int GL_QUADS
    int GL_PROJECTION
    int GL_MODELVIEW

    
    void glTranslatef(GLfloat x, GLfloat y, GLfloat z)
    void glPushMatrix()
    void glPopMatrix()
    void glPushAttrib(GLbitfield mask)
    void glPopAttrib()
    void glEnable(GLenum cap)
    void glDepthFunc(GLenum func)
    void glClearDepth(GLclampd depth)
    void glClear(GLbitfield mask)
    void glFrontFace(GLenum mode)
    void glShadeModel(GLenum mode)
    void glCullFace(GLenum mode)
    void glClearColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha)
    void glBlendFunc(GLenum sfactor, GLenum dfactor)
    void glRotatef(GLfloat angle, GLfloat x, GLfloat y, GLfloat z)
    void glScalef(GLfloat x, GLfloat y, GLfloat z)
    void glBindTexture(GLenum target, GLuint texture)
    void glColor4f(GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha)
    void glBegin(GLenum mode)
    void glNormal3f(GLfloat nx, GLfloat ny, GLfloat nz)
    void glTexCoord2f(GLfloat s, GLfloat t)
    void glVertex3f(GLfloat x, GLfloat y, GLfloat z)
    void glVertex2f(GLfloat x, GLfloat y)
    void glEnd()
    void glCallList(GLuint list)
    void glViewport(GLint x, GLint y, GLsizei width, GLsizei height)
    void glMatrixMode(GLenum mode)
    void glLoadIdentity()

cdef extern from "GL/glu.h":
    void gluPerspective(GLdouble fovy, GLdouble aspect, GLdouble zNear, GLdouble zFar)
    
def initaccel():
    pass
        
class Render(qgl.render.Visitor):
    """
    Traverse the graph and render using OpenGL.
    """
    def push_state(self, node):
        glPushMatrix()
        glPushAttrib(GL_CURRENT_BIT|GL_TEXTURE_BIT|GL_ENABLE_BIT|GL_LIGHTING_BIT|GL_POLYGON_BIT)

    def pop_state(self):
        glPopAttrib()
        glPopMatrix()
    
    def visit_Root(self, node):
        glEnable(GL_DEPTH_TEST)
        glDepthFunc(GL_LEQUAL)
        glClearDepth(1.0)
        glFrontFace(GL_CW)
        glShadeModel(GL_SMOOTH)
        glCullFace(GL_BACK)
        glClearColor(node.background_color[0],node.background_color[1],node.background_color[2],node.background_color[3])
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
        
    def visit_Transform(self, node):
        glTranslatef(node.translate[0],node.translate[1],node.translate[1])
        glRotatef(node.angle, node.axis[0],node.axis[1],node.axis[2])
        glScalef(node.scale[0],node.scale[1],node.scale[2])
        
    def visit_Group(self, node):
        glTranslatef(node.translate[0],node.translate[1],node.translate[1])
        glRotatef(node.angle, node.axis[0],node.axis[1],node.axis[2])
        glScalef(node.scale[0],node.scale[1],node.scale[2])
        for leaf in node.leaves:
            if isinstance(leaf, qgl.scene.Texture):
                glEnable(GL_TEXTURE_2D)
                glBindTexture(GL_TEXTURE_2D, leaf.texture.id)
            elif isinstance(leaf, qgl.scene.Color):
                glColor4f(leaf.rgba[0], leaf.rgba[1], leaf.rgba[2], leaf.rgba[3])
            elif isinstance(leaf, qgl.scene.Quad):
                v = leaf.vertices
                t = leaf.texture_coords
                glBegin(GL_QUADS)
                glNormal3f(0.0,0.0,1.0)
                glTexCoord2f(t[0][0],t[0][1])
                glVertex2f(v[0][0],v[0][1])
                glTexCoord2f(t[1][0],t[1][1])
                glVertex2f(v[1][0],v[1][1])
                glTexCoord2f(t[2][0],t[2][1])
                glVertex2f(v[2][0],v[2][1])
                glTexCoord2f(t[3][0],t[3][1])
                glVertex2f(v[3][0],v[3][1])
                glEnd()
            elif isinstance(leaf, qgl.scene.Sequence):
                v = leaf.vertices
                t = leaf.texture.coords[leaf.frame]
                glEnable(GL_TEXTURE_2D)
                glBindTexture(GL_TEXTURE_2D, leaf.texture.id)
                glBegin(GL_QUADS)
                glNormal3f(0.0,0.0,1.0)
                glTexCoord2f(t[0][0], t[0][1])
                glVertex2f(v[0][0],v[0][1])
                glTexCoord2f(t[1][0],t[1][1])
                glVertex2f(v[1][0],v[1][1])
                glTexCoord2f(t[2][0],t[2][1])
                glVertex2f(v[2][0],v[2][1])
                glTexCoord2f(t[3][0],t[3][1])
                glVertex2f(v[3][0],v[3][1])
                glEnd()
            elif isinstance(leaf, qgl.scene.Text):
                glCallList(leaf.list)
            #elif isinstance(leaf, qgl.scene.ParticleSystem):
            #    glEnable(GL_VERTEX_ARRAY)
            #    glEnable(GL_COLOR_ARRAY)
            #    glVertexPointerf(leaf.positions)
            #    glColorPointerf(leaf.colors)
            #    glDrawArrays(GL_POINTS, 0, leaf.count)
                
    def visit_Viewport(self, node):
        glViewport(node.screen_dimensions[0],node.screen_dimensions[1],node.screen_dimensions[2],node.screen_dimensions[3])
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        gluPerspective(45.0, node.aspect, 0.1, 100)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
